/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FullWizardController.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.fullwizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.ow2.jonas.autostart.preferences.PreferencesFrameDescriptor;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class is responsible for reacting to events generated by pushing any of
 * the three buttons, 'Next', 'Previous', and 'Cancel.' Based on what the
 * controller will update the model to show a new panel and reset the state of
 * the buttons as necessary.
 * @author skabore
 */
public class FullWizardController implements ActionListener {

    /**
     * The wizard.
     */
    private FullWizard wizard;

    /**
     * This constructor accepts a reference to the FullWizard component that
     * created it, which it uses to update the button components and access the
     * FullWizardModel.
     * @param wizard A callback to the FullWizard component that created this
     *        controller.
     */
    public FullWizardController(final FullWizard wizard) {
        this.wizard = wizard;
    }

    /**
     * Calling method for the action listener interface. This class listens for
     * actions performed by the buttons in the FullWizard class, and calls
     * methods below to determine the correct course of action.
     * @param evt The ActionEvent that occurred.
     */
    public void actionPerformed(final ActionEvent evt) {
        if (evt.getActionCommand().equals(FullWizard.CANCEL_BUTTON_ACTION_COMMAND)) {
            cancelButtonPressed();
        } else if (evt.getActionCommand().equals(FullWizard.BACK_BUTTON_ACTION_COMMAND)) {
            backButtonPressed();
        } else if (evt.getActionCommand().equals(FullWizard.NEXT_BUTTON_ACTION_COMMAND)) {
            nextButtonPressed();
        } else if (evt.getActionCommand().equals(FullWizard.CONFIG_BUTTON_ACTION_COMMAND)) {
            configButtonPressed();
        } else if (evt.getActionCommand().equals(FullWizard.PREFERENCES_BUTTON_ACTION_COMMAND)) {
            preferencesButtonPressed();
        } else if (evt.getActionCommand().equals(FullWizard.STOP_BUTTON_ACTION_COMMAND)) {
            stopButtonPressed();
        }
    }

    /**
     * Set the current panel when 'cacel' button is pressed.
     */
    private void cancelButtonPressed() {
        deleteWorkDirectory();
        System.exit(FullWizard.CANCEL_RETURN_CODE);
    }

    /**
     * Set the current panel when 'next' button is pressed.
     */
    private void nextButtonPressed() {

        FullWizardModel model = wizard.getModel();
        FullWizardPanelDescriptor descriptor = model.getCurrentPanelDescriptor();

        // If it is a finishable panel, close down the dialog. Otherwise,
        // get the ID that the current panel identifies as the next panel,
        // and display it.

        Object nextPanelDescriptor = descriptor.getNextPanelDescriptor();

        if (nextPanelDescriptor instanceof FullWizardPanelDescriptor.FinishIdentifier) {
            System.exit(FullWizard.FINISH_RETURN_CODE);
        } else {
            wizard.setCurrentPanel(nextPanelDescriptor);
        }
    }

    /**
     * Set the current panel when 'back' button is pressed.
     */
    private void backButtonPressed() {
        FullWizardModel model = wizard.getModel();
        FullWizardPanelDescriptor descriptor = model.getCurrentPanelDescriptor();
        Object backPanelDescriptor = descriptor.getBackPanelDescriptor();
        wizard.setCurrentPanel(backPanelDescriptor);
    }

    /**
     * Set the current panel when 'config' button is pressed.
     */
    private void configButtonPressed() {
        FullWizardModel model = wizard.getModel();
        FullWizardPanelDescriptor descriptor = model.getCurrentPanelDescriptor();

        Object configPanelDescriptor = descriptor.getConfigPanelDescriptor();
        wizard.setCurrentPanel(configPanelDescriptor);
    }

    /**
     * Set the current panel when 'preferences' button is pressed.
     */
    private void preferencesButtonPressed() {
        if (wizard.getPreferencesFrameDescriptor() == null) {
            wizard.setPreferencesFrameDescriptor(new PreferencesFrameDescriptor(wizard.getBuilder()));
        } else {
            wizard.getPreferencesFrameDescriptor().getPreferencesFrame().initProxyValues();
            wizard.getPreferencesFrameDescriptor().getPreferencesFrame().setVisible(true);
        }
    }

    /**
     * Set the current panel when 'Stop' button is pressed.
     */
    private void stopButtonPressed() {
        if (wizard.getStarter().jonasIsRunning()) {
            if (wizard.getStarter().isValidRequiredParameters()) {
                wizard.getStarter().getOutput().write("");
                new Thread() {

                    @Override
                    public void run() {
                        wizard.getStarter().getOutput().write("Stopping" + " JOnAS server...");
                        wizard.getStarter().stopJonas();
                        wizard.getStopButton().setEnabled(false);
                        wizard.getNextButton().setEnabled(true);
                    }
                }.start();
            } else {
                wizard.showMessage("Please set a valid JOnAS Root folder" + " before stopping server !");
            }
        }
    }

    /**
     * Reset buttons to panel rules.
     */
    public void resetButtonsToPanelRules() {
        FullWizardModel model = wizard.getModel();
        FullWizardPanelDescriptor descriptor = model.getCurrentPanelDescriptor();

        model.setCancelButtonText(FullWizard.CANCEL_TEXT);

        model.setBackButtonText(FullWizard.BACK_TEXT);

        if (descriptor.getBackPanelDescriptor() != null) {
            model.setBackButtonEnabled(Boolean.TRUE);
        } else {
            model.setBackButtonEnabled(Boolean.FALSE);
        }

        if (descriptor.getNextPanelDescriptor() != null) {
            model.setNextFinishButtonEnabled(Boolean.TRUE);
        } else {
            model.setNextFinishButtonEnabled(Boolean.FALSE);
        }

        if (descriptor.getNextPanelDescriptor() instanceof FullWizardPanelDescriptor.FinishIdentifier) {
            model.setNextFinishButtonText(FullWizard.FINISH_TEXT);
        } else {
            model.setNextFinishButtonText(FullWizard.NEXT_TEXT);
        }
    }

    /**
     * Delete the working directory.
     */
    private void deleteWorkDirectory() {
        if (wizard.getStarter() != null && wizard.getStarter().getWorkdirectory() != null) {
            Utility.deleteDirectory(wizard.getStarter().getWorkdirectory());
        } else if (wizard.getBuilder() != null && wizard.getBuilder().getWorkdirectory() != null) {
            Utility.deleteDirectory(wizard.getBuilder().getWorkdirectory());
        }
    }
}
