/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FullWizardModel.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.fullwizard;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;

import org.ow2.jonas.autostart.wizard.WizardPanelNotFoundException;

/**
 * The model for the Wizard component, which tracks the text, icons, and enabled
 * state of each of the buttons, as well as the current panel that is displayed.
 * Note that the model, in its current form, is not intended to be subclassed.
 * @author skabore
 */
public class FullWizardModel {

    /**
     * Identification string for the current panel.
     */
    public static final String CURRENT_PANEL_DESCRIPTOR_PROPERTY = "currentPanelDescriptorProperty";

    /**
     * Property identification String for the Back button's text.
     */
    public static final String BACK_BUTTON_TEXT_PROPERTY = "backButtonTextProperty";

    /**
     * Property identification String for the Back button's enabled state.
     */
    public static final String BACK_BUTTON_ENABLED_PROPERTY = "backButtonEnabledProperty";

    /**
     * Property identification String for the Next button's text.
     */
    public static final String NEXT_FINISH_BUTTON_TEXT_PROPERTY = "nextButtonTextProperty";

    /**
     * Property identification String for the Next button's enabled state.
     */
    public static final String NEXT_FINISH_BUTTON_ENABLED_PROPERTY = "nextButtonEnabledProperty";

    /**
     * Property identification String for the Cancel button's text.
     */
    public static final String CANCEL_BUTTON_TEXT_PROPERTY = "cancelButtonTextProperty";

    /**
     * Property identification String for the Cancel button's enabled state.
     */
    public static final String CANCEL_BUTTON_ENABLED_PROPERTY = "cancelButtonEnabledProperty";

    /**
     * Property identification String for the Set config button's text.
     */
    public static final String CONFIG_BUTTON_TEXT_PROPERTY = "configButtonTextProperty";

    /**
     * Property identification String for the Set config button's enabled state.
     */
    public static final String CONFIG_BUTTON_ENABLED_PROPERTY = "configButtonEnabledProperty";

    /**
     * Property identification String for the Set preferences button's text.
     */
    public static final String PREFERENCES_BUTTON_TEXT_PROPERTY = "preferencesButtonTextProperty";

    /**
     * Property identification String for the Set preferences button's enabled
     * state.
     */
    public static final String PREFERENCES_BUTTON_ENABLED_PROPERTY = "preferencesButtonEnabledProperty";

    /**
     * Property identification String for the Set preferences button's text.
     */
    public static final String STOP_BUTTON_TEXT_PROPERTY = "stopButtonTextProperty";

    /**
     * Property identification String for the Set stop button's enabled state.
     */
    public static final String STOP_BUTTON_ENABLED_PROPERTY = "stopButtonEnabledProperty";

    /**
     * The current panel.
     */
    private FullWizardPanelDescriptor currentPanel;

    /**
     * The panel hashmap.
     */
    private HashMap panelHashmap;

    /**
     * The boutton text hashmap.
     */
    private HashMap buttonTextHashmap;

    /**
     * The boutton icon hashmap.
     */
    private HashMap buttonIconHashmap;

    /**
     * The boutton enabled hashmap.
     */
    private HashMap buttonEnabledHashmap;

    /**
     * The property change support.
     */
    private PropertyChangeSupport propertyChangeSupport;

    /**
     * Default constructor.
     */
    public FullWizardModel() {

        panelHashmap = new HashMap();

        buttonTextHashmap = new HashMap();
        buttonIconHashmap = new HashMap();
        buttonEnabledHashmap = new HashMap();

        propertyChangeSupport = new PropertyChangeSupport(this);

    }

    /**
     * Returns the currently displayed FullWizardPanelDescriptor.
     * @return The currently displayed FullWizardPanelDescriptor
     */
    public FullWizardPanelDescriptor getCurrentPanelDescriptor() {
        return currentPanel;
    }

    /**
     * Registers the FullWizardPanelDescriptor in the model using the
     * Object-identifier specified.
     * @param id Object-based identifier
     * @param descriptor FullWizardPanelDescriptor that describes the panel
     */
    public void registerPanel(final Object id, final FullWizardPanelDescriptor descriptor) {
        panelHashmap.put(id, descriptor);
    }

    /**
     * Sets the current panel to that identified by the Object passed in.
     * @param id Object-based panel identifier
     * @return boolean indicating success or failure
     */
    public boolean setCurrentPanel(final Object id) {
        FullWizardPanelDescriptor nextPanel = (FullWizardPanelDescriptor) panelHashmap.get(id);

        if (nextPanel == null) {
            throw new WizardPanelNotFoundException();
        }

        FullWizardPanelDescriptor oldPanel = currentPanel;
        currentPanel = nextPanel;

        if (oldPanel != currentPanel) {
            firePropertyChange(CURRENT_PANEL_DESCRIPTOR_PROPERTY, oldPanel, currentPanel);
        }
        return true;
    }

    /**
     * @return backButtonText.
     */
    public Object getBackButtonText() {
        return buttonTextHashmap.get(BACK_BUTTON_TEXT_PROPERTY);
    }

    /**
     * Set the backButtonText.
     * @param newText the new text.
     */
    public void setBackButtonText(final Object newText) {

        Object oldText = getBackButtonText();
        if (!newText.equals(oldText)) {
            buttonTextHashmap.put(BACK_BUTTON_TEXT_PROPERTY, newText);
            firePropertyChange(BACK_BUTTON_TEXT_PROPERTY, oldText, newText);
        }
    }

    /**
     * @return nextFinishButtonText.
     */
    public Object getNextFinishButtonText() {
        return buttonTextHashmap.get(NEXT_FINISH_BUTTON_TEXT_PROPERTY);
    }

    /**
     * Set the nextFinishButtonText.
     * @param newText the new text.
     */
    public void setNextFinishButtonText(final Object newText) {

        Object oldText = getNextFinishButtonText();
        if (!newText.equals(oldText)) {
            buttonTextHashmap.put(NEXT_FINISH_BUTTON_TEXT_PROPERTY, newText);
            firePropertyChange(NEXT_FINISH_BUTTON_TEXT_PROPERTY, oldText, newText);
        }
    }

    /**
     * @return cancelButtonText.
     */
    public Object getCancelButtonText() {
        return buttonTextHashmap.get(CANCEL_BUTTON_TEXT_PROPERTY);
    }

    /**
     * Set the cancelButtonText.
     * @param newText the new text.
     */
    public void setCancelButtonText(final Object newText) {

        Object oldText = getCancelButtonText();
        if (!newText.equals(oldText)) {
            buttonTextHashmap.put(CANCEL_BUTTON_TEXT_PROPERTY, newText);
            firePropertyChange(CANCEL_BUTTON_TEXT_PROPERTY, oldText, newText);
        }
    }

    /**
     * @return configButtonText.
     */
    public Object getConfigButtonText() {
        return buttonTextHashmap.get(CONFIG_BUTTON_TEXT_PROPERTY);
    }

    /**
     * Set the configButtonText.
     * @param newText the new text.
     */
    public void setConfigButtonText(final Object newText) {

        Object oldText = getConfigButtonText();
        if (!newText.equals(oldText)) {
            buttonTextHashmap.put(CONFIG_BUTTON_TEXT_PROPERTY, newText);
            firePropertyChange(CONFIG_BUTTON_TEXT_PROPERTY, oldText, newText);
        }
    }

    /**
     * @return preferencesButtonText.
     */
    public Object getPreferencesButtonText() {
        return buttonTextHashmap.get(CONFIG_BUTTON_TEXT_PROPERTY);
    }

    /**
     * Set the preferencesButtonText.
     * @param newText the new text.
     */
    public void setPreferencesButtonText(final Object newText) {

        Object oldText = getPreferencesButtonText();
        if (!newText.equals(oldText)) {
            buttonTextHashmap.put(CONFIG_BUTTON_TEXT_PROPERTY, newText);
            firePropertyChange(CONFIG_BUTTON_TEXT_PROPERTY, oldText, newText);
        }
    }

    /**
     * @return stopButtonText.
     */
    public Object getStopButtonText() {
        return buttonTextHashmap.get(STOP_BUTTON_TEXT_PROPERTY);
    }

    /**
     * Set the stopButtonText.
     * @param newText the new text.
     */
    public void setStopButtonText(final Object newText) {

        Object oldText = getStopButtonText();
        if (!newText.equals(oldText)) {
            buttonTextHashmap.put(STOP_BUTTON_TEXT_PROPERTY, newText);
            firePropertyChange(STOP_BUTTON_TEXT_PROPERTY, oldText, newText);
        }
    }

    /**
     * @return backButtonEnabled.
     */
    public Boolean getBackButtonEnabled() {
        return (Boolean) buttonEnabledHashmap.get(BACK_BUTTON_ENABLED_PROPERTY);
    }

    /**
     * Set the backButtonEnabled.
     * @param newValue the new value.
     */
    public void setBackButtonEnabled(final Boolean newValue) {

        Boolean oldValue = getBackButtonEnabled();
        if (newValue != oldValue) {
            buttonEnabledHashmap.put(BACK_BUTTON_ENABLED_PROPERTY, newValue);
            firePropertyChange(BACK_BUTTON_ENABLED_PROPERTY, oldValue, newValue);
        }
    }

    /**
     * @return nextFinishButtonEnabled.
     */
    public Boolean getNextFinishButtonEnabled() {
        return (Boolean) buttonEnabledHashmap.get(NEXT_FINISH_BUTTON_ENABLED_PROPERTY);
    }

    /**
     * Set the nextFinishButtonEnabled.
     * @param newValue the new value.
     */
    public void setNextFinishButtonEnabled(final Boolean newValue) {

        Boolean oldValue = getNextFinishButtonEnabled();
        if (newValue != oldValue) {
            buttonEnabledHashmap.put(NEXT_FINISH_BUTTON_ENABLED_PROPERTY, newValue);
            firePropertyChange(NEXT_FINISH_BUTTON_ENABLED_PROPERTY, oldValue, newValue);
        }
    }

    /**
     * @return cancelButtonEnabled.
     */
    public Boolean getCancelButtonEnabled() {
        return (Boolean) buttonEnabledHashmap.get(CANCEL_BUTTON_ENABLED_PROPERTY);
    }

    /**
     * Set the cancelButtonEnabled.
     * @param newValue the new value.
     */
    public void setCancelButtonEnabled(final Boolean newValue) {

        Boolean oldValue = getCancelButtonEnabled();
        if (newValue != oldValue) {
            buttonEnabledHashmap.put(CANCEL_BUTTON_ENABLED_PROPERTY, newValue);
            firePropertyChange(CANCEL_BUTTON_ENABLED_PROPERTY, oldValue, newValue);
        }
    }

    /**
     * @return configButtonEnabled.
     */
    public Boolean getConfigButtonEnabled() {
        return (Boolean) buttonEnabledHashmap.get(CONFIG_BUTTON_ENABLED_PROPERTY);
    }

    /**
     * Set the configButtonEnabled.
     * @param newValue the new value.
     */
    public void setConfigButtonEnabled(final Boolean newValue) {

        Boolean oldValue = getConfigButtonEnabled();
        if (newValue != oldValue) {
            buttonEnabledHashmap.put(CONFIG_BUTTON_ENABLED_PROPERTY, newValue);
            firePropertyChange(CONFIG_BUTTON_ENABLED_PROPERTY, oldValue, newValue);
        }
    }

    /**
     * @return preferencesButtonEnabled.
     */
    public Boolean getPreferencesButtonEnabled() {
        return (Boolean) buttonEnabledHashmap.get(CONFIG_BUTTON_ENABLED_PROPERTY);
    }

    /**
     * Set the preferencesButtonEnabled.
     * @param newValue the new value.
     */
    public void setPreferencesButtonEnabled(final Boolean newValue) {
    }

    /**
     * @return stopButtonEnabled.
     */
    public Boolean getStopButtonEnabled() {
        return (Boolean) buttonEnabledHashmap.get(STOP_BUTTON_ENABLED_PROPERTY);
    }

    /**
     * Set the stopButtonEnabled.
     * @param newValue the new value.
     */
    public void setStopButtonEnabled(final Boolean newValue) {

        Boolean oldValue = getStopButtonEnabled();
        if (newValue != oldValue) {
            buttonEnabledHashmap.put(STOP_BUTTON_ENABLED_PROPERTY, newValue);
            firePropertyChange(STOP_BUTTON_ENABLED_PROPERTY, oldValue, newValue);
        }
    }

    /**
     * Add propertyChangeListener.
     * @param property the property to add.
     */
    public void addPropertyChangeListener(final PropertyChangeListener property) {
        propertyChangeSupport.addPropertyChangeListener(property);
    }

    /**
     * Remove propertyChangeListener from propertyChangeSupport.
     * @param property the property to add.
     */
    public void removePropertyChangeListener(final PropertyChangeListener property) {
        propertyChangeSupport.removePropertyChangeListener(property);
    }

    /**
     * Fire propertyChangeListener.
     * @param propertyName the property name.
     * @param oldValue the old value.
     * @param newValue the new value.
     */
    protected void firePropertyChange(final String propertyName, final Object oldValue, final Object newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
}
